#!/usr/bin/env node
"use strict";

var _mri = _interopRequireDefault(require("mri"));

var _execa = _interopRequireDefault(require("execa"));

var fs = _interopRequireWildcard(require("fs-extra"));

var path = _interopRequireWildcard(require("path"));

var _chalk = _interopRequireDefault(require("chalk"));

var _changeCase = require("change-case");

var _prompts = _interopRequireDefault(require("prompts"));

var _resolveAsBin = _interopRequireDefault(require("resolve-as-bin"));

var _getModularRoot = _interopRequireDefault(require("./getModularRoot"));

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Makes the script crash on unhandled rejections instead of silently
// ignoring them. In the future, promise rejections that are not handled will
// terminate the Node.js process with a non-zero exit code.
// See https://github.com/facebook/create-react-app/blob/f36d61a/packages/react-scripts/bin/react-scripts.js#L11-L16
process.on('unhandledRejection', err => {
  throw err;
});
const cracoBin = (0, _resolveAsBin.default)('craco');
const cracoConfig = path.join(__dirname, '..', 'craco.config.js');

function execSync(file, args, options = {
  log: true
}) {
  const {
    log,
    ...opts
  } = options;

  if (log) {
    console.log(_chalk.default.grey(`$ ${file} ${args.join(' ')}`));
  }

  return _execa.default.sync(file, args, {
    stdin: process.stdin,
    stderr: process.stderr,
    stdout: process.stdout,
    ...opts
  });
}

function isYarnInstalled() {
  try {
    _execa.default.sync('yarnpkg', ['-v']);

    return true;
  } catch (err) {
    return false;
  }
}

function isModularType(dir, type) {
  const packageJsonPath = path.join(dir, 'package.json');

  if (fs.existsSync(packageJsonPath)) {
    var _packageJson$modular;

    const packageJson = fs.readJsonSync(packageJsonPath);
    return ((_packageJson$modular = packageJson.modular) === null || _packageJson$modular === void 0 ? void 0 : _packageJson$modular.type) === type;
  }

  return false;
}

function run() {
  const help = `
  Usage:
    $ modular add <package-name>
    $ modular start
    $ modular build
    $ modular test
`;

  if (isYarnInstalled() === false) {
    console.error('Please install `yarn` before attempting to run `modular-scripts`.');
    process.exit(1);
  }

  const argv = (0, _mri.default)(process.argv.slice(2));
  const command = argv._[0];

  try {
    switch (command) {
      case 'add':
        return addPackage(argv._[1], argv['unstable-type']);

      case 'test':
        return test(process.argv.slice(3));

      case 'start':
        return start(argv._[1]);

      case 'build':
        return build(argv._[1]);

      default:
        console.log(help);
        process.exit(1);
    }
  } catch (err) {
    /* eslint-disable @typescript-eslint/no-unsafe-member-access */
    // When there is an `execa` Error with an `.exitCode` the
    // error message will already have been printed out by the
    // process so we only need to bail with the exit code.
    if (err.exitCode !== undefined) {
      process.exit(err.exitCode);
    }
    /* eslint-enable @typescript-eslint/no-unsafe-member-access */


    throw err;
  }
}

async function addPackage(name, typeArg) {
  const type = typeArg || (await (0, _prompts.default)([{
    name: 'type',
    type: 'select',
    message: `What kind of package is ${name}? Huh?`,
    choices: [{
      title: 'A plain package',
      value: 'package'
    }, {
      title: 'A view within an application',
      value: 'view'
    }, {
      title: 'A standalone application',
      value: 'app'
    }],
    initial: 0
  }])).type;

  if (type === 'app') {
    addApp(name);
    return;
  }

  const modularRoot = (0, _getModularRoot.default)();
  const newPackageName = (0, _changeCase.paramCase)(name);
  const newComponentName = (0, _changeCase.pascalCase)(name);
  const newPackagePath = path.join(modularRoot, 'packages', newPackageName);
  const packageTypePath = path.join(__dirname, '../types', type); // create a new package source folder

  if (fs.existsSync(newPackagePath)) {
    console.error(`The package named ${name} already exists!`);
    process.exit(1);
  }

  fs.mkdirpSync(newPackagePath);
  fs.copySync(packageTypePath, newPackagePath);
  const packageRootFilePaths = fs.readdirSync(newPackagePath, {
    withFileTypes: true
  }).filter(entry => entry.isDirectory() === false).map(file => path.join(newPackagePath, file.name));

  for (const packageFilePath of packageRootFilePaths) {
    fs.writeFileSync(packageFilePath, fs.readFileSync(packageFilePath, 'utf8').replace(/PackageName__/g, newPackageName).replace(/ViewName__/g, newPackageName).replace(/ComponentName__/g, newComponentName));
  }

  execSync('yarnpkg', [], {
    cwd: newPackagePath
  });
}

function addApp(name) {
  const modularRoot = (0, _getModularRoot.default)();
  const packagesPath = path.join(modularRoot, 'packages');
  const appPath = path.join(packagesPath, name);
  const appPackageJsonPath = path.join(appPath, 'package.json');
  const appTemplate = 'file:' + path.join(__dirname, '../cra-template-modular-typescript');

  if (fs.existsSync(appPath)) {
    console.error(`The package named ${name} already exists!`);
    process.exit(1);
  }

  execSync('yarnpkg', ['create', 'react-app', name, '--template', appTemplate], {
    cwd: packagesPath
  });
  fs.removeSync(path.join(appPath, '.gitignore'));
  fs.removeSync(path.join(appPath, '.git')); // Technically a `yarn.lock` file isn't created because
  // the dependencies are installed into Modular's root
  // `yarn.lock` file, and therefore during the installation
  // of the template `npm` is used instead and a `package-lock.json`
  // file is created.
  //
  // See: https://github.com/facebook/create-react-app/blob/2da5517689b7510ff8d8b0148ce372782cb285d7/packages/react-scripts/scripts/init.js#L92

  fs.removeSync(path.join(appPath, 'yarn.lock'));
  fs.removeSync(path.join(appPath, 'package-lock.json'));
  fs.removeSync(path.join(appPath, 'README.md'));
  const appPackageJson = fs.readJsonSync(appPackageJsonPath);
  delete appPackageJson.scripts;
  delete appPackageJson.eslintConfig;

  if (appPackageJson.dependencies !== undefined) {
    delete appPackageJson.dependencies['react-scripts'];
  }

  appPackageJson.private = true;
  appPackageJson.modular = {
    type: 'app'
  };
  fs.writeJsonSync(appPackageJsonPath, appPackageJson, {
    spaces: 2
  });
}

function test(args) {
  const modularRoot = (0, _getModularRoot.default)();
  const appDirent = fs.readdirSync(path.join(modularRoot, 'packages'), {
    withFileTypes: true
  }).find(p => p.isDirectory() && isModularType(path.join(modularRoot, 'packages', p.name), 'app'));

  if (!appDirent) {
    // TODO: this is a ridiculous requirement and we should fix it
    throw new Error('modular cannot run tests without at least one app. Add an app and try again.');
  }

  return execSync(cracoBin, ['test', '--config', cracoConfig, ...args], {
    cwd: path.join(modularRoot, 'packages', appDirent.name),
    log: false,
    // @ts-ignore
    env: {
      MODULAR_ROOT: modularRoot
    }
  });
}

function start(appPath) {
  const modularRoot = (0, _getModularRoot.default)();

  if (!isModularType(path.join(modularRoot, 'packages', appPath), 'app')) {
    throw new Error(`The package at ${appPath} is not a valid modular app.`);
  }

  execSync(cracoBin, ['start', '--config', cracoConfig], {
    cwd: path.join(modularRoot, 'packages', appPath),
    log: false,
    // @ts-ignore
    env: {
      MODULAR_ROOT: modularRoot
    }
  });
}

function build(appPath) {
  const modularRoot = (0, _getModularRoot.default)();

  if (!isModularType(path.join(modularRoot, 'packages', appPath), 'app')) {
    throw new Error(`The package at ${appPath} is not a valid modular app.`);
  }

  execSync(cracoBin, ['build', '--config', cracoConfig], {
    cwd: path.join(modularRoot, 'packages', appPath),
    log: false,
    // @ts-ignore
    env: {
      MODULAR_ROOT: modularRoot
    }
  });
}

try {
  void run();
} catch (err) {
  console.error(err);
}